/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.jsr181;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.Iterator;

import javax.xml.stream.XMLStreamReader;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMNamespace;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.axiom.soap.SOAPBody;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axis2.AxisFault;

/**
 * An Apache AXIOM Helper
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since 1.0
 * 
 */
public class AxiomHelper {

    /**
     * 
     * @param factory
     * @param envelope
     * @param jbiPayload
     * @return
     * @throws AxisFault
     */
    public static SOAPBody fillSOAPBodyFromSource(final SOAPFactory factory,
            final SOAPEnvelope envelope, final Source jbiPayload) throws AxisFault {
        SOAPBody body = null;

        try {
            final XMLStreamReader parser = StaxUtils.createXMLStreamReader(jbiPayload);
            final StAXOMBuilder builder = new StAXOMBuilder(factory, parser);
            final OMElement bodyContent = builder.getDocumentElement();

            body = factory.createSOAPBody(envelope);
            body.addChild(bodyContent);

        } catch (final Exception e) {
            throw new AxisFault("Error parsing the response from JBI service to a SOAPBody",
                    "Server", e);
        }
        return body;
    }

    /**
     * 
     * @param envelope
     * @return
     * @throws Exception
     */
    public static Source createSource(final SOAPEnvelope envelope) throws Exception {
        if (envelope == null) {
            throw new Exception("Envelope can not be null to create a source");
        }

        Source result = null;
        final SOAPBody body = envelope.getBody();
        final OMNamespace namespace = envelope.getNamespace();
        final Iterator<OMNamespace> envNS = envelope.getAllDeclaredNamespaces();
        final Iterator<OMNamespace> bodyNS = body.getAllDeclaredNamespaces();

        OMElement rootElement = body.getFirstElement();
        if (rootElement != null) {
            rootElement.declareNamespace(namespace);
            while (envNS.hasNext()) {
                rootElement.declareNamespace(envNS.next());
            }
            while (bodyNS.hasNext()) {
                rootElement.declareNamespace(bodyNS.next());
            }

            final ByteArrayOutputStream os = new ByteArrayOutputStream();
            rootElement.serialize(os);

            os.toByteArray();
            result = new StreamSource(new ByteArrayInputStream(os.toByteArray()));
        }
        return result;
    }
}
