/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.jsr181;

import java.io.File;
import java.io.IOException;

import javax.jbi.JBIException;

import org.apache.commons.io.FileUtils;

import static org.ow2.petals.se.jsr181.Constants.Axis2.CONFIGURATION_FILE;
import static org.ow2.petals.se.jsr181.Constants.Axis2.MODULE_ARCHIVE_EXTENSION;
import static org.ow2.petals.se.jsr181.Constants.Axis2.MODULES_PATH;
import static org.ow2.petals.se.jsr181.Constants.Axis2.SERVICES_PATH;

/**
 * The JSR181 component bootstrap.
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since 1.0
 * 
 */
public class Bootstrap extends org.ow2.petals.component.framework.DefaultBootstrap {

    /**
     * 
     */
    public Bootstrap() {
        // NOP
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.Bootstrap#doInit()
     */
    @Override
    protected void doInit() throws Exception {
        super.doInit();
        this.createWorkDirectories();
        this.copyConfigurationFiles();
    }

    /**
     * 
     * @throws JBIException
     */
    protected void copyConfigurationFiles() throws JBIException {

        final File installRootFile = new File(this.installContext.getInstallRoot());

        /* copy required files from META-INF directory */
        final File modules = new File(installRootFile, MODULES_PATH);
        final File[] metaInfFiles = new File(installRootFile, "META-INF").listFiles();

        for (final File file : metaInfFiles) {
            // copy modules archives
            // FIXME : To be deleted when rampart module will be included as
            // real dependency, for now it is not since it have SNAPSHOT
            // dependencies.
            if (file.getName().endsWith("." + MODULE_ARCHIVE_EXTENSION)) {
                final File destFile = new File(modules, file.getName());
                if (!destFile.exists()) {
                    try {
                        FileUtils.copyFile(file, destFile);
                    } catch (final IOException e) {
                        // do nothing
                    }
                }
            }

            // copy axis2.xml file
            if (file.getName().equals(CONFIGURATION_FILE)) {
                final File destFile = new File(this.installContext.getInstallRoot(), file.getName());
                if (!destFile.exists()) {
                    try {
                        FileUtils.copyFile(file, destFile);
                    } catch (final IOException e) {
                        throw new JBIException(
                                "Can not copy the axis2 configuration file, axis2 will not start properly");
                    }
                }
            }
        }

        // copy modules from component root path
        final File[] rootFiles = installRootFile.listFiles();
        for (final File file : rootFiles) {
            if (file.getName().endsWith("." + MODULE_ARCHIVE_EXTENSION)) {
                final File destFile = new File(modules, file.getName());
                if (!destFile.exists()) {
                    try {
                        FileUtils.copyFile(file, destFile);
                    } catch (final IOException e) {
                        this.getLogger().warning(
                                "The module '" + file.getName()
                                        + "' has not been copied to modules directory");
                        // do nothing
                    }
                }
            }
        }
    }

    /**
     * 
     */
    protected void createWorkDirectories() {
        final File modules = new File(this.installContext.getInstallRoot(), MODULES_PATH);
        final File services = new File(this.installContext.getInstallRoot(), SERVICES_PATH);
        modules.mkdirs();
        services.mkdirs();
    }

}
