/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.jsr181;

import java.io.File;
import java.util.logging.Level;

import javax.jbi.JBIException;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.ow2.petals.component.framework.se.AbstractServiceEngine;
import org.w3c.dom.Document;

import static org.ow2.petals.se.jsr181.Constants.Axis2.CONFIGURATION_FILE;

/**
 * The JSR181 Component
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since 1.0
 * 
 */
public class Component extends AbstractServiceEngine {

    /**
     * The Axis2 configuration context
     */
    private ConfigurationContext axisContext;

    /**
     * The component implementation context
     */
    private Context jsr181Context;

    /**
     * The component implementation deployer
     */
    private Axis2Deployer deployer;

    /**
     * Creates a new instance of {@link Component}
     */
    public Component() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.AbstractComponent#doInit()
     */
    @Override
    protected void doInit() throws JBIException {

        // the axis2.xml is required
        final File axis2File = new File(this.getContext().getInstallRoot(), CONFIGURATION_FILE);
        if (!axis2File.exists()) {
            throw new JBIException(
                    "Can not get the Axis2 configuration file from component context");
        }

        // get axis configuration context
        this.getLogger().fine("Creating Axis configuration context...");
        try {
            axisContext = ConfigurationContextFactory.createConfigurationContextFromFileSystem(this
                    .getContext().getInstallRoot(), axis2File.getAbsolutePath());

            this.getLogger().fine("Axis configuration context created.");
        } catch (final AxisFault e) {
            this.getLogger().log(Level.WARNING, e.getMessage());
            throw new JBIException("Can not initialize JSR181 SE", e);
        }

        this.deployer = new Axis2Deployer(this);
        this.jsr181Context = new Context();

        // set service unit listener
        final SUListener suListener = new SUListener(this);
        this.setServiceUnitListener(suListener);
    }

    /*
     * FIXME : generation does not work at runtime. This is because Axis2 needs
     * tools.jar lib which is not available (???) in classloader... For now the
     * WSDL is generated during SU packaging from the Service class and is
     * injected in the JBI descriptor.
     */
    @Override
    public Document getServiceDescription(ServiceEndpoint endpoint) {
        return super.getServiceDescription(endpoint);
        // Document result = null;
        // ServiceUnitDataHandler suDataHandler = ((ServiceUnitManager)
        // this.getServiceUnitManager())
        // .getSUDataHandlerForEndpoint(endpoint);
        // List<Provides> provideList =
        // suDataHandler.getDescriptor().getServices().getProvides();
        // Iterator<Provides> iter = provideList.iterator();
        // boolean found = false;
        // AxisService service = null;
        // while (iter.hasNext() && !found) {
        // Provides provides = iter.next();
        // service = this.jsr181Context.getService(provides);
        // found = (service != null);
        // }
        //
        // try {
        // ByteArrayOutputStream bos = new ByteArrayOutputStream();
        // service.printWSDL(bos);
        // result = XMLUtil.createDocumentFromString(bos.toString());
        // } catch (AxisFault e) {
        // this.getLogger().log(Level.WARNING, "Can not get the AxisService
        // description", e);
        // result = super.getServiceDescription(endpoint);
        // }
        // return result;
    }

    /**
     * 
     * @return
     */
    public ConfigurationContext getAxisContext() {
        return this.axisContext;
    }

    /**
     * 
     * @return
     */
    public Context getJSR181Context() {
        return this.jsr181Context;
    }

    /**
     * 
     * @return
     */
    public Axis2Deployer getAxis2Deployer() {
        return this.deployer;
    }
}
