/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.jsr181;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.axis2.AxisFault;
import org.apache.axis2.engine.AxisConfiguration;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.su.AbstractServiceUnitListener;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;
import org.ow2.petals.component.framework.su.ServiceUnitManager;
import org.ow2.petals.component.framework.util.ClassLoaderUtil;

/**
 * The JSR181 Component Service Unit listener. It creates the service at SU
 * deployment.
 * 
 * @author Christophe HAMERLING - eBM WebSourcing
 * @since 1.0
 * 
 */
public class SUListener extends AbstractServiceUnitListener {

    /**
     * The Service deployer
     */
    private Axis2Deployer deployer;

    /**
     * The JBI Component
     */
    private Component component;

    /**
     * The Component context
     */
    private Context jsr181Context;

    /**
     * The logger
     */
    private Logger logger;

    /**
     * Creates a new instance of {@link SUListener}
     * 
     * @param component
     */
    public SUListener(Component component) {
        this.component = component;
        this.deployer = this.component.getAxis2Deployer();
        this.jsr181Context = this.component.getJSR181Context();
        this.logger = this.component.getLogger();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.su.AbstractServiceUnitListener#onSUDeployed(java.lang.String,
     *      java.lang.String,
     *      org.ow2.petals.component.framework.jbidescriptor.generated.Jbi)
     */
    @Override
    public void onSUDeployed(final String serviceUnitName, final String suRootPath,
            final Jbi descriptor) throws PEtALSCDKException {
        // FIXME : Do it in the Start phase, the CDK need to provide a better API
        
        final List<Provides> providesList = descriptor.getServices().getProvides();
        ServiceUnitDataHandler suDatahandler = null;
        for (final Provides provides : providesList) {
            if (suDatahandler == null) {
                suDatahandler = ((ServiceUnitManager) this.component.getServiceUnitManager())
                        .getSUDataHandlerForService(provides);
                ConfigurationExtensions extensions = suDatahandler
                        .getConfigurationExtensions(provides);
                String className = extensions.get("class");

                // create a JBI URL, not used but required...
                URL serviceURL = null;
                try {
                    serviceURL = new URL("http://localhost/services/" + provides.getServiceName());
                } catch (MalformedURLException e) {
                    this.logger.log(Level.WARNING, e.getMessage());
                }

                this.logger.log(Level.INFO, "Creating Axis Service for class '" + className + "'");

                ClassLoader cl = new URLClassLoader(ClassLoaderUtil.getUrls(suRootPath),
                        Component.class.getClassLoader());
                ClassLoader saved = Thread.currentThread().getContextClassLoader();
                AxisService service = null;
                try {
                    Thread.currentThread().setContextClassLoader(cl);
                    service = deployer.deployJAXWS(className, cl, provides, serviceURL);
                } finally {
                    Thread.currentThread().setContextClassLoader(saved);
                }

                // add the service to the context
                if (service != null) {
                    this.jsr181Context.addService(provides, service);
                } else {
                    throw new PEtALSCDKException("Can not create the Axis Service");
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.su.AbstractServiceUnitListener#onSUUndeployed(java.lang.String,
     *      java.lang.String,
     *      org.ow2.petals.component.framework.jbidescriptor.generated.Jbi)
     */
    @Override
    public void onSUUndeployed(String serviceUnitName, String suRootPath, Jbi descriptor)
            throws PEtALSCDKException {
        // FIXME : Must be done in the stop phase...
        final AxisConfiguration config = this.component.getAxisContext().getAxisConfiguration();
        final List<Provides> providesList = descriptor.getServices().getProvides();
        for (final Provides provides : providesList) {
            AxisService service = this.jsr181Context.getService(provides);
            if (service != null) {
                this.logger.log(Level.INFO, "Removing service '" + service.getName()
                        + "' from Axis2 context");
                try {
                    config.removeServiceGroup(service.getName());
                    service.getAxisConfiguration().removeService(service.getName());
                } catch (AxisFault e) {
                    this.logger.log(Level.WARNING, e.getMessage());
                } finally {
                    // remove from cache
                    this.jsr181Context.removeService(provides);
                }
            } else {
                this.logger.log(Level.WARNING, "Service '" + service.getName()
                        + "' not found, can not be unregistered from Axis2");
            }
        }
    }
}
